"
COPYRIGHT (c) 2020-2021 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

Object subclass:#RGObject
	instanceVariableNames:'properties name parent'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGObject comment:'I am the root class of the Ring meta-model definitions.
A Ring definition has a name, properties and knows its environment'
!

!RGObject class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020-2021 LabWare


"
! !

!RGObject class methodsFor:'instance creation'!

asYetUnclassifiedProtocolName
    ^ Compiler asYetUncategorizedMethodCategory

    "Modified (format): / 04-02-2021 / 10:09:50 / Jan Vrany <jan.vrany@labware.com>"
!

named: aString

    "create model in its own environment"

    ^self new
        pvtName: aString;
        yourself
!

named: aName parent: anRGObject

    ^self basicNew
        parent: anRGObject;
        initialize;
        pvtName: aName asSymbol;
        yourself
!

new
    "return an initialized instance"

    ^ self basicNew initialize.
!

parent: anRGObject

    ^self basicNew
        parent: anRGObject;
        initialize;
        yourself
!

unnamed

    ^self new
!

unresolved

    ^ self basicNew
        initializeUnresolved;
        yourself.
!

unresolvedNamed: aString withParent: anRGDefintion

    ^self basicNew
        initializeUnresolved; 
        pvtName: aString asSymbol;
        parent: anRGDefintion;
        yourself
!

unresolvedWithParent: anRGDefintion

    ^self basicNew
        initializeUnresolved; 
        parent: anRGDefintion;
        yourself
! !

!RGObject methodsFor:'accessing'!

ask

    ^ self 
!

backend
    
    ^ self environment backend
!

environment
    
    <ignoreForMutations>

    ^ self propertyNamed: #environment ifAbsent: [ self parent environment ].
!

fullName

    ^self name
!

name

    ^ (self backend nameFor: self) orDefaultForUnresolved 
!

name: aString

    | oldName |
    
    oldName := name.
    self backend setNameFor: self to: aString.
    self parent child: self renamedFrom: oldName to: aString.
!

package

    ^ self parent package
!

parent
    "The parent of a class definition element: method, comment and variable is the class definition. This method retrieves the class that defines such element"
    
    <ignoreForMutations>
    parent ifNil: [ 
        self parent: self defaultParentStub.
        self addoptToParentStub ].
    
    ^ parent.
!

parent: anRGBehavior

    "If possible, do not use this method directly."
    <ignoreForMutations>

    parent := anRGBehavior.
    "self addoptToParentStub."

    
!

resolvedNameOrNil

    ^ self hasResolvedName
        ifTrue: [ self name. ]
        ifFalse: [ nil ].		
!

unresolveName

    | oldName result |
    
    oldName := name.
    result := self unresolvedValue: self unresolvedName.
    self pvtName: result.
    self parent child: self renamedFrom: oldName to: result value.
    ^ result
! !

!RGObject methodsFor:'announcements'!

announce: anAnnouncement

    self environment announcer announce: anAnnouncement
!

announcer

    ^ self environment announcer
! !

!RGObject methodsFor:'converting'!

asRGDefinition

    ^ self
!

unresolvedValue

    ^ RGUnresolvedValue new.
!

unresolvedValue: aDefaultValue

    ^ RGUnresolvedValue new
        default: aDefaultValue;
        yourself
! !

!RGObject methodsFor:'copying'!

copyForBehaviorDefinition

    "returns copy of the behavior model that contains only information related to the behavior 
    definition"
    <ignoreForMutations> "removing of ^ can cause infinite loops"
    ^ self shallowCopy copyForBehaviorDefinitionPostCopy
!

copyForBehaviorDefinitionPostCopy

    <ignoreForMutations>

    super postCopy.
    
    
    
    
!

copyWithParent: newParent

    "create copy of the definition and then assign the new parent"
    
    ^ self copy 
        parent: newParent;
        yourself.
!

fullCopy

    ^ self copy.
!

postCopy

    super postCopy.
    
    properties := properties copy.
! !

!RGObject methodsFor:'defaults'!

unresolvedName

    ^ #unresolved
! !

!RGObject methodsFor:'events'!

child: aChildDefinition renamedFrom: oldName to: newName

    "react on renaming of a child definition. By default do nothing"
! !

!RGObject methodsFor:'hierarchy-defaults'!

defaultClassStub

    <ignoreForMutations>
    ^ RGClass unresolved
!

defaultCommentStubIn: anRGBehavior

    <ignoreForMutations>
    ^ RGComment unresolved
        parent: anRGBehavior;
        yourself.
!

defaultEnvironmentStub

    <ignoreForMutations>
    ^ RGEnvironment unresolved
!

defaultFixedLayoutStub

    <ignoreForMutations>
    ^ RGFixedLayout unresolved
!

defaultFixedLayoutStubIn: anRGBehavior

    <ignoreForMutations>
    ^ RGFixedLayout unresolved
        parent: anRGBehavior;
        yourself.
!

defaultPackageStubIn: anRGEnvironment

    <ignoreForMutations>
    ^ RGPackage unresolved
        parent: anRGEnvironment;
        yourself.
!

defaultTraitCompositionStub

    ^ RGTraitComposition unresolved
!

defaultTraitCompositionStubIn: anRGBehavior
    <ignoreForMutations>

    ^ RGTraitComposition unresolved
        parent: anRGBehavior;
        yourself.
! !

!RGObject methodsFor:'initialization'!

initialize

    super initialize.

    name := self unresolvedValue: self unresolvedName.

!

initializeUnresolved

    self propertyNamed: #resolved put: false. 
    name := self unresolvedValue: self unresolvedName.
    

! !

!RGObject methodsFor:'managing container'!

addoptToParentStub
!

defaultParentStub

    self subclassResponsibility
! !

!RGObject methodsFor:'private'!

pvtParent: anRGObject

    parent := anRGObject.
! !

!RGObject methodsFor:'private - backend interface'!

pvtFullyResolved

    ^ (self pvtResolvableProperties collect: [:each | each value]) allSatisfy: [:each | each isRingFullyResolved]
!

pvtFullyUnresolved		
        
    ^ (self pvtResolvableProperties collect: [:each | each value]) noneSatisfy: [:each | each isRingFullyResolved ]
!

pvtName

    ^ name
!

pvtName: aString

     name := aString
!

pvtResolvableProperties

    ^ { #name -> name }.
!

pvtResolvedProperties

    ^ self pvtResolvableProperties select: [:each | each value isRingResolved ] thenCollect: #key
!

pvtUnresolvedProperties

    ^ self pvtResolvableProperties select: [:each | each value isRingResolved not ] thenCollect: #key
! !

!RGObject methodsFor:'properties'!

copyPropertiesFrom: anRGObject

    properties := anRGObject properties copy.
!

hasProperties

    ^ properties isEmptyOrNil not
!

hasProperty: propertyName

    ^ self hasProperties
        ifTrue: [ properties includesKey: propertyName ]	
        ifFalse:[ false ].
!

hasPropertyNamed: propertyName

    self
        deprecated: 'Use hasProperty: instead'
        transformWith: '`@rec hasPropertyNamed: `@arg' -> '`@rec hasProperty: `@arg'.

    ^ self hasProperty: propertyName
!

properties

    ^ properties ifNil: [ properties := IdentityDictionary new ]
!

propertyNamed: propertyName 
    
    ^ self
        propertyNamed: propertyName
        ifAbsent: [ nil ]
!

propertyNamed: annotationName ifAbsent: exceptionBlock 

    ^ self properties
            at: annotationName
            ifAbsent: [ exceptionBlock value ]
!

propertyNamed: annotationName ifAbsentPut: blockValue 

    ^ self properties
        at: annotationName
        ifAbsentPut: blockValue
!

propertyNamed: annotationName put: value 

    self properties
        at: annotationName
        put: value
!

propertyNames
    
    self hasProperties
        ifFalse:[ ^ OrderedCollection new ].
    ^ properties keys
!

removePropertyNamed: propertyName

    (self hasProperty: propertyName) ifTrue: [ 
        properties removeKey: propertyName ]
! !

!RGObject methodsFor:'resolving'!

hasResolved: propertySymbol

    ^ self resolvedProperties includes: propertySymbol


!

hasResolvedAll: aCollectionOfSymbols

    ^ self resolvedProperties includesAll: aCollectionOfSymbols


!

hasUnresolved: propertySymbol

    ^ (self resolvedProperties includes: propertySymbol) not


!

hasUnresolvedAll: aCollectionOfSymbols

    ^ (self resolvedProperties includesAll: aCollectionOfSymbols) not


!

isRingFullyResolved

    ^ self backend hasFullyResolved: self
!

isRingFullyUnresolved		
    
    ^ self backend hasFullyUnresolved: self 		

!

makeResolved

    self markAsRingResolved.
    
    name := self name markAsRingResolved.
!

markAsRingResolved

    self propertyNamed: #resolved put: true.
    
!

resolvedProperties

    ^ (self backend resolvedPropertiesFor: self)
!

unresolvedProperties

    ^ (self backend unresolvedPropertiesFor: self)
! !

!RGObject methodsFor:'testing'!

hasResolvedName		
    
    ^ self hasResolved: #name
!

isMeta

    ^ false
!

isRGObject

    ^true
!

isRingResolved

    ^ self propertyNamed: #resolved ifAbsent: [ true ]
! !

!RGObject methodsFor:'testing types'!

isBehavior

    ^ false
!

isClass

    ^ false
!

isComment

    ^false
!

isEnvironment

    ^false
!

isGlobalVariable

    ^false
!

isLayout

    ^false
!

isMetaclass

    ^ false
!

isMetaclassTrait

    ^ false
!

isMethod

    ^false
!

isPackage

    ^false
!

isProtocol

    ^false
!

isSlot

    ^ false
!

isTraitAlias

    ^ false
!

isTraitComposition

    ^ false
!

isTraitExclusion

    ^ false
!

isTraitTransformation

    ^ false
!

isVariable

    "is the definition a variable class?"

    ^ false
!

isVariableDefinition
    self 
        deprecated: 'Use #isVariable instead.' 
        transformWith: '`@receiver isVariableDefinition' -> '`@receiver isVariable'.
    
    ^ self isVariable
! !

!RGObject class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

