"{ Package: 'stx:goodies/ring/monticello' }"!

!MCClassDefinition methodsFor:'*Ring-Monticello'!

asRGDefinition

    ^ self ensureRingDefinitionIn: RGEnvironment new
! !

!MCClassDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment

    ^ anRGEnvironment backend definitionFor: self ifAbsentRegister: [
        | def metaclassDef |
        def := anRGEnvironment ensureClassNamed: self className asSymbol.
        metaclassDef := anRGEnvironment ensureClassNamed: (self className, ' class') asSymbol.
        def metaclass: metaclassDef.
        def traitComposition: (self traitCompositionFrom: self traitComposition parent: def).			
        metaclassDef traitComposition: (self traitCompositionFrom: self classTraitComposition parent: metaclassDef).			
        (self superclassName = #nil)
            ifTrue: [ def superclass: def ]
            ifFalse: [ def superclass: (anRGEnvironment ensureClassNamed: superclassName ) ].
        def category: self category.
        def layout: (self ring2LayoutType parent: def).
        metaclassDef layout: (RGFixedLayout parent: metaclassDef). 
        def comment: (
            (RGComment parent: def) 
                content: self comment;
                author: (self authorForStamp: self commentStamp);
                time: (self timeForStamp: self commentStamp);
                yourself).
        self variables do: [ :each |
            (each class type = #instance) ifTrue: [
                (each name includes: $>) 
                    ifFalse: [ def layout addSlot: (RGInstanceVariableSlot named: each name parent: def layout)]
                    ifTrue: [ 
                        | unknownSlot |
                        unknownSlot := RGUnknownSlot named: ((each name copyUpTo: $=) copyWithout: $#) trimBoth parent: def layout.
                        unknownSlot expression: (each name copyAfter: $>) trimBoth.
                        def layout addSlot: unknownSlot]].
            (each class type = #class) ifTrue: [ 
                def addClassVariable: (RGClassVariable named: each name parent: def)].
            (each class type = #classInstance) ifTrue: [ 
                metaclassDef layout addSlot: (RGInstanceVariableSlot named: each name parent: metaclassDef layout)].
            (each class type = #pool) ifTrue: [ 
                def addSharedPool: (RGPoolVariable named: each name parent: def)]. ]. 
        def]. 
    
! !

!MCClassDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment package: anRGPackage
    
    | def |
    
    def := self ensureRingDefinitionIn: anRGEnvironment.
    def package: anRGPackage.
    def category: def tags first. "fix tags to do not contain package names"
    ^ def
! !

!MCClassDefinition methodsFor:'*Ring-Monticello'!

ring2LayoutType

    ^ (Dictionary newFrom: {    
        "/ No, do not treat compiled method specially. They're special
        "/ only in OSVM-derived smalltalks.
        #compiledMethod -> "RGCompiledMethodLayout"RGFixedLayout. 
        #bytes -> RGByteLayout. 
        #immediate -> RGImmediateLayout. 
        #words -> RGWordLayout. 
        #week -> RGWeakLayout.
        #variable -> RGVariableLayout.
        #ephemeron -> RGEphemeronLayout.
        #normal -> RGFixedLayout.
        #weak -> RGWeakLayout.
    }) at: type.

    "Modified: / 23-12-2020 / 14:43:09 / Jan Vrany <jan.vrany@labware.com>"
! !

!MCClassTraitDefinition methodsFor:'*Ring-Monticello'!

asRGDefinition

    ^ self ensureRingDefinitionIn: RGEnvironment new
! !

!MCClassTraitDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment

    ^ anRGEnvironment backend definitionFor: self ifAbsentRegister: [
        | def traitDef |
        def := anRGEnvironment ensureMetaclassTraitNamed: (self baseTrait, ' classTrait') asSymbol.
        traitDef := anRGEnvironment ensureTraitNamed: (self baseTrait) asSymbol.
        def baseTrait: traitDef.
        traitDef classTrait: def.
        def traitComposition: (self traitCompositionFrom: self classTraitComposition parent: def).
        traitDef category: self category. "makes no sense for metaclass trait"
        def]. 
    
! !

!MCDefinition methodsFor:'*Ring-Monticello'!

authorForStamp: stamp
    
    ^ stamp isEmptyOrNil
        ifTrue: [ '<historical>']
        ifFalse: [ stamp substrings first].
! !

!MCDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment

    "do nothing by default"
! !

!MCDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment package: anRGPackage
    
    "by default do not set the package"

    ^ self ensureRingDefinitionIn: anRGEnvironment.
    
! !

!MCDefinition methodsFor:'*Ring-Monticello'!

timeForStamp: stamp

    ^ stamp isEmptyOrNil
        ifTrue:  [ Timestamp epoch]
        ifFalse: [ ([(Timestamp fromString: stamp) asTime] on: Exception do: [Timestamp epoch]) ].

    "Modified: / 23-12-2020 / 13:49:37 / Jan Vrany <jan.vrany@labware.com>"
! !

!MCDefinition methodsFor:'*Ring-Monticello'!

traitCompositionFrom: aStringOrNil parent: aBehavior
    
    | composition transformations |
    
    composition := RGTraitComposition parent: aBehavior.

    aStringOrNil ifNil: [ ^ composition ].

    transformations := RGTraitCompositionVisitor new parse: aStringOrNil for: composition.
    transformations do: [ :each |
        composition addTransformation: each ].
    ^ composition
! !

!MCMethodDefinition methodsFor:'*Ring-Monticello'!

asRGDefinition

"	method2 := (Object>>#isRingObject) asRingDefinition.
    self assert: method2 class equals: RGMethod.	
    self assert: method2 selector equals: #isRingObject.
    self assert: method2 parent name equals: #Object.

    self deny: (method2 environment == env1)."

    ^ self ensureRingDefinitionIn: RGEnvironment new
    
! !

!MCMethodDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment

    ^ anRGEnvironment backend definitionFor: self ifAbsentRegister: [
        | def classDef metaclassDef parent protocol |
        classDef := anRGEnvironment ensureClassNamed: self className asSymbol.
        
        classDef isTrait 
            ifFalse: [ 
                metaclassDef := anRGEnvironment ensureClassNamed: (self className, ' class') asSymbol.
                classDef metaclass: metaclassDef.
                metaclassDef baseClass: classDef.]
            ifTrue: [ 
                metaclassDef := anRGEnvironment ensureMetaclassTraitNamed: (self className, ' classTrait') asSymbol.
                classDef classTrait: metaclassDef.
                metaclassDef baseTrait. classDef.].
                
        parent := self classIsMeta 
            ifFalse: [ classDef ]
            ifTrue: [ metaclassDef ].
        
        def := RGMethod named: self selector parent: parent.
        parent addLocalMethod: def.
            
        def sourceCode: self source asStringWithNativeLineEndings.
        def author: (self authorForStamp: self timeStamp).
        def time: (self timeForStamp: self timeStamp).
        
        protocol := parent ensureProtocolNamed: self category asSymbol.
        parent addProtocol: protocol.
        def protocol: protocol.
        def 
    ]

    "Modified: / 04-02-2021 / 09:14:46 / Jan Vrany <jan.vrany@labware.com>"
! !

!MCMethodDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment package: anRGPackage
    
    | def  |
    
    def := self ensureRingDefinitionIn: anRGEnvironment.
    def package: anRGPackage.

    ^ def
! !

!MCSnapshot methodsFor:'*Ring-Monticello'!

importInto: anRGEnvironment asPackageNamed: aPackageName

    | anRGPacakgeDefinition |
    anRGPacakgeDefinition := anRGEnvironment ensurePackageNamed: aPackageName.
    self definitions do: [ :each |
        each ensureRingDefinitionIn: anRGEnvironment package: anRGPacakgeDefinition ].

    ^ anRGPacakgeDefinition
! !

!MCTraitDefinition methodsFor:'*Ring-Monticello'!

ensureRingDefinitionIn: anRGEnvironment

    ^ anRGEnvironment backend definitionFor: self ifAbsentRegister: [
        | def classTraitDef |
        def := anRGEnvironment ensureTraitNamed: self className asSymbol.
        classTraitDef := anRGEnvironment ensureMetaclassTraitNamed: (self className, ' classTrait') asSymbol.
        def classTrait: classTraitDef.
        classTraitDef baseTrait: def.
        def traitComposition: (self traitCompositionFrom: self traitComposition parent: def).
"		classTraitDef traitComposition: (self traitCompositionFrom: self classTraitComposition parent: classTraitDef)".		
        ((self superclassName = #nil) or: [ self superclassName isNil "for traits"])
            ifTrue: [ def superclass: def ]
            ifFalse: [ def superclass: (anRGEnvironment ensureClassNamed: superclassName ) ].
        def category: self category.
        def comment: (
            (RGComment parent: def) 
                content: self comment;
                author: (self authorForStamp: self commentStamp);
                time: (self timeForStamp: self commentStamp);
                yourself).
        def]. 
    
! !

!RGBehaviorStrategyUser methodsFor:'*Ring-Monticello'!

asMCDefinition

    ^ self behaviorStrategy asMCDefinition
! !

!RGByteLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #bytes 
! !

!RGClassStrategy methodsFor:'*Ring-Monticello'!

asMCDefinition

    ^ MCClassDefinition 
        name: self owner name
        superclassName: (self owner superclass ifNil: [ 'nil' ] ifNotNil: [ :aSuperclass | aSuperclass name ])
        traitComposition: self owner traitCompositionString 
        classTraitComposition: self owner metaclass traitCompositionString
        category: self owner category 
        instVarNames: self instVarNames
        classVarNames: self owner classVarNames
        poolDictionaryNames: self owner sharedPoolNames
        classInstVarNames: self owner metaclass instVarNames
        type: self mcType
        comment: self owner comment content
        commentStamp: self owner comment stamp
! !

!RGClassStrategy methodsFor:'*Ring-Monticello'!

classLayout
    "TODO"
    ^ layout"
        ifNil: [ 
            | superLayout scope |
            superLayout := superclass classLayout.
            scope := superLayout slotScope extend.
            layout := superLayout class extending: superLayout scope: scope host: self ]"
! !

!RGClassStrategy methodsFor:'*Ring-Monticello'!

classPool

    "TODO"
    ^ Dictionary newFrom: (self classVarNames collect: [ :each | each -> nil ])
! !

!RGClassStrategy methodsFor:'*Ring-Monticello'!

mcType

    ^ self layout mcType
! !

!RGClassStrategy methodsFor:'*Ring-Monticello'!

referencedPackages

    ^ Array with: self package.
! !

!RGComment methodsFor:'*Ring-Monticello'!

stamp

    "TODO:"
    ^ self author
! !

!RGCompiledMethodLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #compiledMethod 
! !

!RGEnvironment methodsFor:'*Ring-Monticello'!

writeIntoTonel: aFileReference

    (self ask packages select: #isRingResolved) do: [ :aPackage |
        | aSnapshot info aVersion |
        aSnapshot := aPackage asMCSnapshot.

        info := MCVersionInfo new.
        info instVarNamed: #name put: aPackage name, '-author'.
        aVersion := MCVersion package: (MCPackage new name: aPackage name) info: info snapshot: aSnapshot dependencies: #().
        TonelWriter 
            fileOut: (aVersion)
            on: aFileReference ]		
! !

!RGEphemeronLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #ephemeron 
! !

!RGFixedLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #normal 
! !

!RGImmediateLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #immediate 
! !

!RGMethod methodsFor:'*Ring-Monticello'!

asMCDefinition

    ^ MCMethodDefinition 
        className: self parent theNonMetaClass name
        classIsMeta: self parent isMeta
        selector: self name
        category: self protocol
        timeStamp: ''
        source: self sourceCode
! !

!RGPackage methodsFor:'*Ring-Monticello'!

asMCSnapshot

    | definitions | 
    
    definitions := ((((self definedBehaviors reject: [:each | each isMeta]) sorted: [:a :b | a name < b name ]) collect: [:each | each asMCDefinition]), (self extensionMethods collect: [:each | each asMCDefinition])) asOrderedCollection.
    definitions addAll: ((self definedBehaviors, (self definedBehaviors collect: [:each | each classSide])) flatCollect: [ :behavior |
        (behavior localMethods reject: [:each | (each  parent instanceSide package = each package) not])
            collect: [ :method | method asMCDefinition] ]).
    
    ^ MCSnapshot fromDefinitions: definitions.
! !

!RGTraitV2Strategy methodsFor:'*Ring-Monticello'!

asMCDefinition

    ^ MCClassDefinition 
        name: self name
        superclassName: (self superclass ifNil: [ 'nil' ] ifNotNil: [ :aSuperclass | aSuperclass name ])
        traitComposition: self owner traitCompositionString 
        classTraitComposition: self owner metaclass traitCompositionString
        category: self category 
        instVarNames: self instVarNames
        classVarNames: self classVarNames
        poolDictionaryNames: self sharedPoolNames
        classInstVarNames: self metaclass instVarNames
        type: self mcType
        comment: self comment content
        commentStamp: self comment stamp
! !

!RGTraitV2Strategy methodsFor:'*Ring-Monticello'!

classLayout
    "TODO"
    ^ layout"
        ifNil: [ 
            | superLayout scope |
            superLayout := superclass classLayout.
            scope := superLayout slotScope extend.
            layout := superLayout class extending: superLayout scope: scope host: self ]"
! !

!RGTraitV2Strategy methodsFor:'*Ring-Monticello'!

classPool

    "TODO"
    ^ Dictionary newFrom: (self classVarNames collect: [ :each | each -> nil ])
! !

!RGTraitV2Strategy methodsFor:'*Ring-Monticello'!

mcType

    ^ self layout mcType
! !

!RGTraitV2Strategy methodsFor:'*Ring-Monticello'!

referencedPackages

    ^ Array with: self package.
! !

!RGVariableLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #variable 
! !

!RGWeakLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #weak 
! !

!RGWordLayout methodsFor:'*Ring-Monticello'!

mcType

    ^ #words 
! !

!TonelRepository methodsFor:'*Ring-Monticello'!

asRingEnvironmentWith: packageNames

    ^ self asRingEnvironmentWith: packageNames cleaning: true
! !

!TonelRepository methodsFor:'*Ring-Monticello'!

asRingEnvironmentWith: packageNames cleaning: aBoolen

    | aRG2Environment |
    aRG2Environment := RGEnvironment new.
    aRG2Environment announcer suspendAllWhile: [ 
        (packageNames) do: [ :pn | 
            | snap defs package version |
            package := aRG2Environment ensurePackageNamed: pn asSymbol.
            version := self versionWithInfo: (self versionInfoFromVersionNamed: pn).
            snap := version snapshot.

            defs := snap definitions.
            defs do: [ :each |			
                each ensureRingDefinitionIn: aRG2Environment package: package ]
            displayingProgress: 'snap'.] 
         displayingProgress: 'version'.
        ].		
            
    aBoolen ifTrue: [ 
        aRG2Environment clean ].
    
    aRG2Environment behaviorsDo: [ :beh |
        beh superclass == beh ifTrue: [ beh superclass: nil. ] ].			
    ^ aRG2Environment
! !

!stx_goodies_ring_monticello class methodsFor:'documentation'!

extensionsVersion_HG

    ^ '$Changeset: <not expanded> $'
! !
